/*
 * Copyright 2012-2017 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.common.yaml;

import com.google.gson.Gson;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.redhat.thermostat.common.json.JsonUtil;
import com.redhat.thermostat.common.json.models.SwaggerDef;
import org.junit.Test;

import java.io.File;
import java.io.IOException;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

public class YamlToJsonTest {

    private static final String API1_YAML_FILE = "/systems-swagger.yaml";

    @Test
    public void readYamlTest() throws IOException {
        final File yamlFile1 = new File(getClass().getResource(API1_YAML_FILE).getFile());
        YamlToJson y2j = new YamlToJson();
        JsonObject yaml = y2j.yamlToJsonObject(yamlFile1);
        assertNotNull(yaml);
        JsonElement getDescr = JsonUtil.fetch(yaml, "paths./systems/{systemId}.get.description");
        assertNotNull(getDescr);
        assertTrue(getDescr.isJsonPrimitive());
        assertEquals("Get information for system {systemId}.", getDescr.getAsString());
    }

    @Test
    public void yamlToModelTest() throws IOException {
        YamlToJson y2j = new YamlToJson();
        final File yamlFile = new File(getClass().getResource(API1_YAML_FILE).getFile());
        JsonObject yaml = y2j.yamlToJsonObject(yamlFile);
        assertNotNull(yaml);

        // resolve all refs
        RefResolver.resolveRefs(yaml, yaml);

        // convert JSON to actual Java class
        Gson gson = new Gson();
        SwaggerDef swaggerDef = gson.fromJson(yaml, SwaggerDef.class);

        final String getdescr = JsonUtil.fetch(yaml, "paths./systems/{systemId}.get.description").getAsString();
        assertEquals(getdescr, swaggerDef.getPaths().get("/systems/{systemId}").getGETAPI().getDescription());
    }

    private static final String GPL = "GPL v2 with Classpath Exception";

    private static final String YAML_WITH_NULL =
            "info:\n" +
            "  title: Thermostat Web Gateway JVM Information API\n" +
            "  license:\n" +
            "    name: " + GPL + "\n" +
            "    url: 'http://www.gnu.org/licenses'\n" +
            "    org:  null\n" +
            "    phone: 416-363-6097\n" +
            "consumes:\n" +
            "  - application/json\n" +
            "produces:\n" +
            "  - application/json\n" +
            "  - text/html; charset=utf-8\n" +
            "basePath: /jvms/0.0.1\n";

    @Test
    public void testNull() throws IOException {
        YamlToJson y2j = new YamlToJson();
        JsonObject yaml = y2j.yamlToJsonObject(YAML_WITH_NULL);
        assertEquals(GPL, JsonUtil.fetch(yaml, "info.license.name").getAsString());
        final JsonElement nell = JsonUtil.fetch(yaml, "info.license.org");
        assertTrue(nell.isJsonNull());
    }
}
